@@ -360,6 +360,8 @@ GSList *         pango_layout_get_lines_readonly   (PangoLayout    *layout);
  * Flags that influence the behavior of [method@Pango.Layout.serialize].
  *
  * New members may be added to this enumeration over time.
+ *
+ * Since: 1.50
  */
 typedef enum {
   PANGO_LAYOUT_SERIALIZE_DEFAULT = 0,
@@ -382,20 +384,18 @@ gboolean        pango_layout_write_to_file         (PangoLayout
 
 /**
  * PangoLayoutDeserializeError:
- * @PANGO_LAYOUT_SERIALIZE_INVALID: Unspecified error
- * @PANGO_LAYOUT_SERIALIZE_INVALID_SYNTAX: The serialized data had
- *   the wrong structure (e.g. a member was expected to be a JSon object,
- *   but was an array)
- * @PANGO_LAYOUT_SERIALIZE_INVALID_VALUE: A JSon value could not be
+ * @PANGO_LAYOUT_DESERIALIZE_INVALID: Unspecified error
+ * @PANGO_LAYOUT_DESERIALIZE_INVALID_VALUE: A JSon value could not be
  *   interpreted
- * @PANGO_LAYOUT_SERIALIZE_MISSING_VALUE: A required JSon member was
+ * @PANGO_LAYOUT_DESERIALIZE_MISSING_VALUE: A required JSon member was
  *   not found
  *
  * Errors that can be returned by [func@Pango.Layout.deserialize].
+ *
+ * Since: 1.50
  */
 typedef enum {
   PANGO_LAYOUT_DESERIALIZE_INVALID,
-  PANGO_LAYOUT_DESERIALIZE_INVALID_SYNTAX,
   PANGO_LAYOUT_DESERIALIZE_INVALID_VALUE,
   PANGO_LAYOUT_DESERIALIZE_MISSING_VALUE,
 } PangoLayoutDeserializeError;
@@ -409,9 +409,11 @@ GQuark          pango_layout_deserialize_error_quark (void);
  * @PANGO_LAYOUT_DESERIALIZE_CONTEXT: Apply context information
  *   from the serialization to the `PangoContext`
  *
- * Flags that influence the behavior of [method@Pango.Layout.deserialize].
+ * Flags that influence the behavior of [func@Pango.Layout.deserialize].
  *
  * New members may be added to this enumeration over time.
+ *
+ * Since: 1.50
  */
 typedef enum {
   PANGO_LAYOUT_DESERIALIZE_DEFAULT = 0,
