@@ -35,6 +35,7 @@
 gboolean opt_display = TRUE;
 int opt_dpi = 96;
 gboolean opt_pixels = FALSE;
+gboolean opt_pango_units = FALSE;
 const char *opt_font = "";
 gboolean opt_header = FALSE;
 const char *opt_output = NULL;
@@ -76,6 +77,8 @@ guint16 opt_fg_alpha = 65535;
 gboolean opt_bg_set = FALSE;
 PangoColor opt_bg_color = {65535, 65535, 65535};
 guint16 opt_bg_alpha = 65535;
+gboolean opt_serialized = FALSE;
+const char *file_arg;
 
 /* Text (or markup) to render */
 static char *text;
@@ -102,6 +105,23 @@ make_layout(PangoContext *context,
   PangoAlignment align;
   PangoLayout *layout;
 
+  if (opt_serialized)
+    {
+      char *text;
+      gsize len;
+      GBytes *bytes;
+      GError *error = NULL;
+
+      if (!g_file_get_contents (file_arg, &text, &len, &error))
+        fail ("%s\n", error->message);
+      bytes = g_bytes_new_take (text, size);
+      layout = pango_layout_deserialize (context, bytes, PANGO_LAYOUT_DESERIALIZE_CONTEXT, &error);
+      if (!layout)
+        fail ("%s\n", error->message);
+      g_bytes_unref (bytes);
+      return layout;
+    }
+
   layout = pango_layout_new (context);
   if (opt_markup)
     pango_layout_set_markup (layout, text, -1);
@@ -119,20 +139,38 @@ make_layout(PangoContext *context,
   if (size > 0)
     pango_font_description_set_size (font_description, size * PANGO_SCALE);
 
-  if (opt_width > 0)
-    pango_layout_set_width (layout, (opt_width * opt_dpi * PANGO_SCALE + 36) / 72);
+  if (opt_width >= 0)
+    {
+      if (opt_pango_units)
+        pango_layout_set_width (layout, opt_width);
+      else
+        pango_layout_set_width (layout, (opt_width * opt_dpi * PANGO_SCALE + 36) / 72);
+    }
 
-  if (opt_height > 0)
-    pango_layout_set_height (layout, (opt_height * opt_dpi * PANGO_SCALE + 36) / 72);
+  if (opt_height >= 0)
+    {
+      if (opt_pango_units)
+        pango_layout_set_width (layout, opt_height);
+      else
+        pango_layout_set_height (layout, (opt_height * opt_dpi * PANGO_SCALE + 36) / 72);
+    }
   else
     pango_layout_set_height (layout, opt_height);
 
   if (opt_indent != 0)
-    pango_layout_set_indent (layout, (opt_indent * opt_dpi * PANGO_SCALE + 36) / 72);
+    {
+      if (opt_pango_units)
+        pango_layout_set_indent (layout, opt_indent);
+      else
+        pango_layout_set_indent (layout, (opt_indent * opt_dpi * PANGO_SCALE + 36) / 72);
+    }
 
   if (opt_spacing != 0)
     {
-      pango_layout_set_spacing (layout, (opt_spacing * opt_dpi * PANGO_SCALE + 36) / 72);
+      if (opt_pango_units)
+        pango_layout_set_spacing (layout, opt_spacing);
+      else
+        pango_layout_set_spacing (layout, (opt_spacing * opt_dpi * PANGO_SCALE + 36) / 72);
       pango_layout_set_line_spacing (layout, 0.0);
     }
   if (opt_line_spacing >= 0.0)
@@ -829,6 +867,8 @@ parse_options (int argc, char *argv[])
      "Deprecated",		      "file"},
     {"pixels",		0, 0, G_OPTION_ARG_NONE,			&opt_pixels,
      "Use pixel units instead of points (sets dpi to 72)",		NULL},
+    {"pango-units",	0, 0, G_OPTION_ARG_NONE,			&opt_pango_units,
+     "Use Pango units instead of points",		                NULL},
     {"rtl",		0, 0, G_OPTION_ARG_NONE,			&opt_rtl,
      "Set base direction to right-to-left",				NULL},
     {"rotate",		0, 0, G_OPTION_ARG_DOUBLE,			&opt_rotate,
@@ -847,6 +887,8 @@ parse_options (int argc, char *argv[])
      "Width in points to which to wrap lines or ellipsize",	    "points"},
     {"wrap",		0, 0, G_OPTION_ARG_CALLBACK,			&parse_wrap,
      "Text wrapping mode (needs a width to be set)",   "word/char/word-char"},
+    {"serialized",       0, 0, G_OPTION_ARG_NONE,                        &opt_serialized,
+     "Create layout from a serialized file",                            NULL},
     {NULL}
   };
   GError *error = NULL;
@@ -890,6 +932,12 @@ parse_options (int argc, char *argv[])
       exit (1);
     }
 
+  if (opt_serialized && argc != 2)
+    {
+      g_printerr ("Usage: %s [OPTION...] FILE\n", g_get_prgname ());
+      exit (1);
+    }
+
   /* set up the backend */
   if (!opt_viewer)
     {
@@ -900,7 +948,13 @@ parse_options (int argc, char *argv[])
 
   /* Get the text
    */
-  if (opt_text)
+  if (opt_serialized)
+    {
+      file_arg = argv[1];
+      text = g_strdup ("");
+      len = 0;
+    }
+  else if (opt_text)
     {
       text = g_strdup (opt_text);
       len = strlen (text);
